﻿using ReportWriterSample.Model;
using Syncfusion.Report;
using Syncfusion.ReportWriter;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Reflection.Emit;
using System.Text;
using System.Threading.Tasks;
using Xamarin.Forms;

namespace ReportWriterSample
{
    public class PDFViewerPage : ContentPage
    {
        Syncfusion.SfPdfViewer.XForms.SfPdfViewer pdfViewerControl = new Syncfusion.SfPdfViewer.XForms.SfPdfViewer();
        Button printButton = new Button();
        Button savePdfButton = new Button();
        MemoryStream pdfStream = null;

        public string Type = null;

        public PDFViewerPage()
        {
            this.Appearing += PDFViewerPage_Appearing;

            printButton.IsEnabled = false;
            printButton.Text = "Print PDF";
            printButton.Clicked += printButton_Clicked;

            savePdfButton.IsEnabled = false;
            savePdfButton.Text = "Save PDF";
            savePdfButton.Clicked += SavePdfButton_Clicked;

            var stackLayout = new StackLayout();
            stackLayout.Children.Add(printButton);
            stackLayout.Children.Add(savePdfButton);
            stackLayout.Children.Add(pdfViewerControl);
            pdfViewerControl.VerticalOptions = LayoutOptions.FillAndExpand;

            Content = stackLayout;
        }

        private async void SavePdfButton_Clicked(object sender, EventArgs e)
        {
            if (pdfStream != null)
            {
                await DependencyService.Get<IDependencyServiceInterface>().SavePdfAsync(pdfStream);
            }
        }

        private async void printButton_Clicked(object sender, EventArgs e)
        {
            var pdfStream = await GetReportStream();

            if (pdfStream != null)
            {
                if (Device.RuntimePlatform == Device.Android)
                {
                    await DisplayAlert("Print", "Printing support need to be implemented", "OK");
                }
                else if (Device.RuntimePlatform == Device.iOS)
                {
                    await DisplayAlert("Print", "Printing support need to be implemented", "OK");
                }
                else
                {
                    await DependencyService.Get<IDependencyServiceInterface>().PrintPdfAsync(pdfStream);
                }
            }
        }

        private async void PDFViewerPage_Appearing(object sender, EventArgs e)
        {
            Stream pdfStream =  await GetReportStream();

            if (pdfStream != null)
            {
                pdfStream.Position = 0;
                try
                {
                    pdfViewerControl.LoadDocument(pdfStream);
                }
                catch(Exception ex)
                {
                    await DisplayAlert("Problem with PDF Viewer", ex.ToString(), "OK");
                }
            }
        }

        private async Task<MemoryStream> GetReportStream()
        {
            if (this.pdfStream != null)
            {
                this.pdfStream.Position = 0;
                return this.pdfStream;
            }

            var assembly = typeof(PDFViewerPage).GetTypeInfo().Assembly;
            Stream fontStream = assembly.GetManifestResourceStream("ReportWriterSample.Font.bigfontsite.com_microsoft_jhenghei.ttf");
            Stream reportStream = null;

            ReportDataSourceCollection collection = new ReportDataSourceCollection();

            if (Type == null)
            {
                reportStream = assembly.GetManifestResourceStream("ReportWriterSample.Reports.Sales Dashboard.rdlc");
                string Year = "2002";
                collection.Add(new ReportDataSource { Name = "TopSalesPerson", Value = SalesPersons.GetTopSalesPerson(int.Parse(Year)) });
                collection.Add(new ReportDataSource { Name = "TopStores", Value = Stores.GetTopStores(int.Parse(Year)) });
                collection.Add(new ReportDataSource { Name = "TopProduct", Value = Products.GetTopProducts(int.Parse(Year)) });
            }


            ReportWriter writer = new ReportWriter();
            writer.ReportProcessingMode = ProcessingMode.Local;
            //writer.PDFOptions = new PDFOptions();
            //writer.PDFOptions.Fonts = new Dictionary<string, Stream>();
            //writer.PDFOptions.Fonts.Add("Microsoft JhengHei", fontStream);

            writer.LoadReport(reportStream);
            writer.DataSources = collection;

            MemoryStream stream = new MemoryStream();
            writer.Save(stream, WriterFormat.PDF);
            this.pdfStream = stream;

            if (this.pdfStream != null)
            {
                this.printButton.IsEnabled = true;
                this.savePdfButton.IsEnabled = true;
                return this.pdfStream;
            }
            else
            {
                await DisplayAlert("Problem with Report Writer", "We have a problem to generate the Report from ReportWriter.", "OK");
            }

            return null;
        }
    }
}
